package meerkat.mixer.proofs;

import com.google.protobuf.Message;
import meerkat.crypto.concrete.ECElGamalEncryption;
import meerkat.crypto.concrete.Util;
import meerkat.protobuf.Crypto;
import meerkat.protobuf.Mixing;
import org.bouncycastle.math.ec.ECPoint;
import org.factcenter.qilin.primitives.concrete.ECGroup;

import java.math.BigInteger;
import java.util.Random;

/**
 * Prover, Simulator and Verifier
 */
public class ECElGamalMixProtocols {
    final ECElGamalMixParams params;
    final ECGroup group;
    final ECElGamalEncryption encryptor;
    final ECPoint g;
    final ECPoint h;
    final Random rand;


    public ECElGamalMixProtocols(ECElGamalMixParams params, Random rand) {
        this.params = params;
        this.rand = rand;
        group = params.group;
        encryptor = params.encryptor;
        g = params.g;
        h = params.h;
    }

    public class DlogStatementSchnorrProver implements SigmaProtocol.Prover {
        ECElGamalMixParams.DlogStatement statement;
        ECElGamalMixParams.DlogStatementWitness witness;

        BigInteger r = null;

        public DlogStatementSchnorrProver(ECElGamalMixParams.DlogStatement statement, ECElGamalMixParams.DlogStatementWitness witness) {
            this.statement = statement;
            this.witness = witness;
        }

        @Override
        public Mixing.Mix2Proof.DlogProof.FirstMessage getFirstMessage() {
            r = encryptor.generateRandomExponent(rand);
            ECPoint gr = group.multiply(statement.g, r);
            ECPoint hr = group.multiply(statement.h, r);

            Mixing.Mix2Proof.DlogProof.FirstMessage firstMessage = Mixing.Mix2Proof.DlogProof.FirstMessage.newBuilder()
                    .setGr(encryptor.encodeElement(gr))
                    .setHr(encryptor.encodeElement(hr))
                    .build();

            return firstMessage;
        }

        @Override
        public Crypto.BigInteger getFinalMessage(BigInteger challenge) {
            return Util.encodeBigInteger(challenge.multiply(witness.x).add(r).mod(group.orderUpperBound()));
        }
    }
}
