package meerkat.mixer.proofs;

import com.google.protobuf.Message;

import java.math.BigInteger;

/**
 * Generic conjuction of sigma protocols
 */
public class SigmaProtocolAnd {
    abstract public class Prover implements SigmaProtocol.Prover {
        final SigmaProtocol.Prover[] provers;

        abstract protected Message buildConcatenatedFirstMessage(Message... firstMessages);
        abstract protected Message buildConcatenatedFinalMessage(Message... finalMessages);

        public Prover(SigmaProtocol.Prover... provers) {
            this.provers = provers;
        }

        @Override
        public Message getFirstMessage() {
            Message[] firstMessages = new Message[provers.length];
            for (int i = 0; i < firstMessages.length; ++i) {
                firstMessages[i] = provers[i].getFirstMessage();
            }
            return buildConcatenatedFirstMessage(firstMessages);
        }

        @Override
        public Message getFinalMessage(BigInteger challenge) {
            Message[] finalMessages = new Message[provers.length];
            for (int i = 0; i < finalMessages.length; ++i) {
                finalMessages[i] = provers[i].getFinalMessage(challenge);
            }
            return buildConcatenatedFinalMessage(finalMessages);
        }
    }

    abstract public class Verifier implements SigmaProtocol.Verifier {
        final SigmaProtocol.Verifier[] verifiers;

        abstract protected Message getFirstMessage(Message concatenated, int verifier);
        abstract protected Message getFinalMessage(Message concatenated, int verifier);

        public Verifier(SigmaProtocol.Verifier... verifiers) {
            this.verifiers = verifiers;
        }

        public boolean verify(Message firstMessage, BigInteger challenge, Message finalMessage) {
            boolean ok = true;

            for (int i = 0; i < verifiers.length; ++i) {
                ok &= verifiers[i].verify(getFirstMessage(firstMessage, i), challenge, getFinalMessage(finalMessage, i));
            }
            return ok;
        }
    }
}
